## $Id: interfaces.py 16786 2022-02-08 21:35:52Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from zope.interface import Attribute, invariant, Invalid
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email)
from waeup.kofa.schema import FormattedDate, TextLineChoice, PhoneNumber
from waeup.kofa.interfaces import IKofaObject
from waeup.kofa.students.vocabularies import (
    nats_vocab, GenderSource, StudyLevelSource)
from waeup.kofa.applicants.interfaces import (
    contextual_reg_num_source,
    IApplicantBaseData,
    IApplicantRefereeReport)
from waeup.kofa.refereeentries import RefereeEntryField
from waeup.kofa.university.vocabularies import StudyModeSource
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    )
from waeup.aaue.interfaces import MessageFactory as _
from waeup.aaue.payments.interfaces import ICustomOnlinePayment

programme_types_vocab = SimpleKofaVocabulary(
    (_('Undergraduate Programme (100 level)'), 'regular'),
    (_('Direct Entry (200 level)'), 'direct'),
    (_('not applicable'), 'na'),
    )

certificate_types_vocab = SimpleKofaVocabulary(
    (_('Full-time Degree'), 'ft'),
    (_('Part-time Degree'), 'pt'),
    (_('Diploma'), 'dp'),
    (_('Masters Degree'), 'ma'),
    (_('Doctorate Degree'), 'phd'),
    )

document_types_vocab = SimpleKofaVocabulary(
    (_('Certificate'), 'certificate'),
    (_('Result'), 'result'),
    (_('Transcript'), 'transcript'),
    (_('Studentship'), 'studship'),
    )

application_types_vocab = SimpleKofaVocabulary(
    (_('Academic Staff Positions'), 'academic'),
    (_('Senior Non-Teaching Staff'), 'senior'),
    (_('Junior Staff (Non-Teaching)'), 'junior'),
    )

request_types_vocab = SimpleKofaVocabulary(
    (_('Scanned Result'), 'result'),
    (_('Scanned Certificate'), 'certificate'),
    (_('Scanned Transcript'), 'transcript'),
    (_('Attestation'), 'attest'),
    (_('Proficiency in English Language'), 'english'),
    )

rating_vocab = SimpleKofaVocabulary(
    (_('Excellent'), 'e'),
    (_('Very good'), 'vg'),
    (_('Good'), 'g'),
    (_('Slightly above average'), 'saa'),
    (_('Average'), 'a'),
    (_('Below average'), 'ba'),
    (_('Unable to assess'), 'unable'),
    )

overallpromise_vocab = SimpleKofaVocabulary(
    (_('Very good (highest 10%)'), 'vg'),
    (_('Good (next 15%)'), 'g'),
    (_('Above average (next 15%)'), 'aa'),
    (_('Average (middle 20%)'), 'a'),
    (_('Below average (lower 10%)'), 'ba'),
    )


class ICustomUGApplicant(IApplicantBaseData):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the UG_OMIT* tuples.
    """

    #programme_type = schema.Choice(
    #    title = _(u'Programme Type'),
    #    vocabulary = programme_types_vocab,
    #    required = False,
    #    )

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = True,
        )

    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )

    perm_address = schema.Text(
        title = _(u'Permanent Address'),
        required = False,
        )

    home_town = schema.TextLine(
        title = _(u'Home Town'),
        required = False,
        )

    #jamb_reg_number = schema.TextLine(
    #    title = _(u'JAMB Registration Number'),
    #    required = False,
    #    )

    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )

    jamb_subjects = schema.Text(
        title = _(u'JAMB Subjects and Scores'),
        required = False,
        )

    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )

    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )

    course3 = schema.Choice(
        title = _(u'3rd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )

    fst_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    fst_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )

    fst_sit_sc_pin = schema.TextLine(
        title = _(u'Scratch Card Pin'),
        required = False,
        readonly = False,
        )

    fst_sit_sc_serial_number = schema.TextLine(
        title = _(u'Scratch Card Serial Number'),
        required = False,
        readonly = False,
        )

    fst_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    fst_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )

    fst_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )

    scd_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    scd_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )

    scd_sit_sc_pin = schema.TextLine(
        title = _(u'Scratch Card Pin'),
        required = False,
        readonly = False,
        )

    scd_sit_sc_serial_number = schema.TextLine(
        title = _(u'Scratch Card Serial Number'),
        required = False,
        readonly = False,
        )

    scd_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    scd_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )

    scd_sit_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )

    alr_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    alr_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = False,
        readonly = False,
        )

    alr_date = FormattedDate(
        title = _(u'Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    alr_results = schema.List(
        title = _(u'Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )

    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )

    hq_type2 = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname2 = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no2 = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree2 = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school2 = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session2 = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc2 = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )

    hq_type3 = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname3 = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no3 = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree3 = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school3 = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session3 = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc3 = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )

    nysc_year = schema.Int(
        title = _(u'Nysc Year'),
        required = False,
        readonly = False,
        )

    nysc_location = schema.TextLine(
        title = _(u'Nysc Location'),
        required = False,
        )

    nysc_lga = schema.Choice(
        source = LGASource(),
        title = _(u'Nysc LGA'),
        required = False,
        )

    employer = schema.TextLine(
        title = _(u'Employer'),
        required = False,
        readonly = False,
        )

    emp_position = schema.TextLine(
        title = _(u'Employer Position'),
        required = False,
        readonly = False,
        )

    emp_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    emp_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    emp_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )

    employer2 = schema.TextLine(
        title = _(u'2nd Employer'),
        required = False,
        readonly = False,
        )

    emp2_position = schema.TextLine(
        title = _(u'2nd Employer Position'),
        required = False,
        readonly = False,
        )

    emp2_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    emp2_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )

    emp2_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )

    former_matric = schema.TextLine(
        title = _(u'If yes, matric number'),
        required = False,
        readonly = False,
        )

    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )


    master_sheet_number = schema.TextLine(
        title = _(u'Master Sheet Number'),
        required = False,
        readonly = False,
        )

    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )

    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )

    screening_score = schema.Int(
        title = _(u'Screening Points'),
        required = False,
        )

    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )

    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        )

    @invariant
    def course_choice(applicant):
        if applicant.course1 == applicant.course2:
            raise Invalid(_("2nd choice course must differ from 1st choice course."))
        if applicant.course1 == applicant.course3:
            raise Invalid(_("3rd choice course must differ from 1st choice course."))
        if applicant.course2 == applicant.course3:
            raise Invalid(_("3rd choice course must differ from 2nd choice course."))

#ICustomUGApplicant['programme_type'].order = IApplicantBaseData[
#    'reg_number'].order


class ICustomPGApplicant(IApplicantBaseData):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = True,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    hq_type2 = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname2 = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no2 = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree2 = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school2 = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session2 = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc2 = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )

    hq_type3 = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname3 = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no3 = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree3 = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school3 = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session3 = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc3 = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )

    hq_type4 = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )

    hq_fname4 = schema.TextLine(
        title = _(u'Full Name'),
        required = False,
        readonly = False,
        )

    hq_matric_no4 = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )

    hq_degree4 = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )

    hq_school4 = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )

    hq_session4 = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )

    hq_disc4 = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    presently_inst = schema.TextLine(
        title = _(u'If yes, name of institution'),
        required = False,
        readonly = False,
        )
    nysc_year = schema.Int(
        title = _(u'Nysc Year'),
        required = False,
        readonly = False,
        )
    nysc_lga = schema.Choice(
        source = LGASource(),
        title = _(u'Nysc Location'),
        description = _(u'Leave blank for exception letters.'),
        required = False,
        )
    employer = schema.TextLine(
        title = _(u'Employer'),
        required = False,
        readonly = False,
        )
    emp_position = schema.TextLine(
        title = _(u'Employer Position'),
        required = False,
        readonly = False,
        )
    emp_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )
    employer2 = schema.TextLine(
        title = _(u'2nd Employer'),
        required = False,
        readonly = False,
        )
    emp2_position = schema.TextLine(
        title = _(u'2nd Employer Position'),
        required = False,
        readonly = False,
        )
    emp2_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp2_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp2_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )
    former_matric = schema.TextLine(
        title = _(u'If yes, matric number'),
        required = False,
        readonly = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        readonly = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Float(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        readonly = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    referees = schema.List(
        title = _(u'Referees'),
        value_type = RefereeEntryField(),
        required = False,
        defaultFactory=list,
        )

class ITranscriptApplicant(IKofaObject):
    """A transcript applicant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Application Id'),
        required = False,
        readonly = False,
        )

    reg_number = TextLineChoice(
        title = _(u'Kofa Registration Number'),
        readonly = False,
        required = True,
        source = contextual_reg_num_source,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        readonly = False,
        required = True,
        )

    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = False,
        #date_format = u'%d/%m/%Y', # Use grok-instance-wide default
        show_year = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    place_of_birth = schema.TextLine(
        title = _(u'Place of Birth'),
        readonly = False,
        required = False,
        )

    nationality = schema.Choice(
        vocabulary = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    perm_address = schema.Text(
        title = _(u'Current Local Address'),
        required = False,
        readonly = False,
        )

    dispatch_address = schema.Text(
        title = _(u'Dispatch Addresses'),
        description = u'Addresses to which transcript should be posted.',
        required = False,
        readonly = False,
        )

    entry_mode = schema.Choice(
        title = _(u'Entry Mode'),
        source = StudyModeSource(),
        required = False,
        readonly = False,
        )

    entry_session = schema.Choice(
        title = _(u'Entry Session'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    end_session = schema.Choice(
        title = _(u'End Session'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    course_studied = schema.Choice(
        title = _(u'Course of Study / Degree'),
        source = CertificateSource(),
        required = False,
        readonly = False,
        )

    purpose = schema.TextLine(
        title = _(u'Purpose of this Application'),
        readonly = False,
        required = False,
        )

    course_changed = schema.Choice(
        title = _(u'Change of Study Course'),
        description = u'If yes, select previous course of study.',
        source = CertificateSource(),
        readonly = False,
        required = False,
        )

    change_level = schema.Choice(
        title = _(u'Change Level'),
        description = u'If yes, select level at which you changed course of study.',
        source = StudyLevelSource(),
        required = False,
        readonly = False,
        )

    no_copies = schema.Choice(
        title = _(u'Number of Copies'),
        description = u'Must correspond with the number of dispatch addresses above.',
        values=[1, 2, 3, 4],
        required = False,
        readonly = False,
        default = 1,
        )

class ICertificateRequest(IKofaObject):
    """A transcript applicant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Application Id'),
        required = False,
        readonly = False,
        )

    reg_number = TextLineChoice(
        title = _(u'Kofa Registration Number'),
        readonly = False,
        required = True,
        source = contextual_reg_num_source,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        readonly = False,
        required = True,
        )

    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = False,
        #date_format = u'%d/%m/%Y', # Use grok-instance-wide default
        show_year = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    place_of_birth = schema.TextLine(
        title = _(u'Place of Birth'),
        readonly = False,
        required = False,
        )

    nationality = schema.Choice(
        vocabulary = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    entry_session = schema.Choice(
        title = _(u'Entry Session'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    end_session = schema.Choice(
        title = _(u'End Session'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    course_studied = schema.Choice(
        title = _(u'Course of Study / Degree'),
        source = CertificateSource(),
        required = True,
        readonly = False,
        )

    certificate_type = schema.Choice(
        title = _(u'Certificate Type'),
        vocabulary = certificate_types_vocab,
        required = False,
        )


class IVerificationRequest(IKofaObject):
    """A applicant asking for verification.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Application Id'),
        required = False,
        readonly = False,
        )

    #reg_number = TextLineChoice(
    #    title = _(u'Kofa Registration Number'),
    #    readonly = False,
    #    required = True,
    #    source = contextual_reg_num_source,
    #    )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Verification Body Reference Number'),
        readonly = False,
        required = True,
        )

    body_address = schema.Text(
        title = _(u'Verification Body Address'),
        required = True,
        )

    document_type = schema.Choice(
        title = _(u'Document Type'),
        vocabulary = document_types_vocab,
        required = True,
        )

class IFedexRequest(IKofaObject):
    """A applicant requests payment for courier.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Application Id'),
        required = False,
        readonly = False,
        )

    trans_id = schema.TextLine(
        title = _(u'Transcript Application Id'),
        required = True,
        readonly = False,
        #description = u'This serve as a unique identifier which '
        #               'allows the the officer to verify the transcript '
        #               'in order to avoid errors before dispatch.',
        )

    #reg_number = TextLineChoice(
    #    title = _(u'Kofa Registration Number'),
    #    readonly = False,
    #    required = True,
    #    source = contextual_reg_num_source,
    #    )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    matric_number = schema.TextLine(
        title = _(u'WES Reference Number (where applicable)'),
        readonly = False,
        required = False,
        )

    dispatch_address = schema.Text(
        title = _(u'Dispatch Address'),
        required = True,
        )

class IRecruitment(IKofaObject):
    """A recruitment application.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Application Id'),
        required = False,
        readonly = False,
        )

    #reg_number = TextLineChoice(
    #    title = _(u'Kofa Registration Number'),
    #    readonly = False,
    #    required = True,
    #    source = contextual_reg_num_source,
    #    )

    application_types = schema.Choice(
        title = _(u'Application Category'),
        vocabulary = application_types_vocab,
        required = True,
        )

    position_comment = schema.Text(
        title = _(u'Desired Position'),
        required = True,
        description = u'Copy and paste the vacant position '
                       'text from the <a target="_blank" '
                       'href="https://aauekpoma.edu.ng/">university website</a>.',
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    address = schema.Text(
        title = _(u'Address'),
        required = True,
        )

class ISendByEmailRequest(IKofaObject):
    """A applicant asking for sending an email.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Applicant Id'),
        required = False,
        readonly = False,
        )

    firstname = schema.TextLine(
        title = _(u'First Name'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Last Name (Surname)'),
        required = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    email = schema.ASCIILine(
        title = _(u"Applicant's Email Address"),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    body_address = schema.Text(
        title = _(u'Address of Requesting Organization'),
        required = True,
        )

    body_email = schema.ASCIILine(
        title = _(u"Email Address of Requesting Organization"),
        required = True,
        constraint=validate_email,
        )

    request_type = schema.Choice(
        title = _(u'Request Type'),
        vocabulary = request_types_vocab,
        required = True,
        )

    document_type = schema.Choice(
        title = _(u'Document Type'),
        vocabulary = document_types_vocab,
        required = True,
        )

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant,
                       ITranscriptApplicant, ICertificateRequest,
                       IVerificationRequest, ISendByEmailRequest,
                       IFedexRequest, IRecruitment):
    """An interface for all types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicatnt data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    #programme_type = schema.Choice(
    #    title = _(u'Programme Type'),
    #    vocabulary = programme_types_vocab,
    #    required = True,
    #    )

    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )

    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )

    course3 = schema.Choice(
        title = _(u'3rd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )

    fst_sit_fname = schema.TextLine(
        title = _(u'Full Name'),
        required = True,
        readonly = False,
        )

    fst_sit_no = schema.TextLine(
        title = _(u'Exam Number'),
        required = True,
        readonly = False,
        )

    fst_sit_sc_pin = schema.TextLine(
        title = _(u'Scratch Card Pin'),
        required = True,
        readonly = False,
        )

    fst_sit_sc_serial_number = schema.TextLine(
        title = _(u'Scratch Card Serial Number'),
        required = True,
        readonly = False,
        )

    fst_sit_date = FormattedDate(
        title = _(u'Exam Date'),
        required = True,
        readonly = False,
        show_year = True,
        )

    fst_sit_type = schema.Choice(
        title = _(u'Exam Type'),
        required = True,
        readonly = False,
        vocabulary = exam_types,
        )

    # course1 works only on manage pages. On edit pages course1 input is missing
    # and no Invalid exception is raised.
    # NoInputData: NoInputD...course1'
    # Therefore, we check and compare course1 on CustomApplicantEditFormPage.
    @invariant
    def course_choice(applicant):
        if applicant.course2 == applicant.course3:
            raise Invalid(_("3rd choice course must differ from 2nd choice course."))

#ICustomUGApplicantEdit['programme_type'].order = ICustomUGApplicant[
#    'programme_type'].order
ICustomUGApplicantEdit['date_of_birth'].order = ICustomUGApplicant[
    'date_of_birth'].order
ICustomUGApplicantEdit['course1'].order = ICustomUGApplicant[
    'course1'].order
ICustomUGApplicantEdit['course2'].order = ICustomUGApplicant[
    'course2'].order
ICustomUGApplicantEdit['course3'].order = ICustomUGApplicant[
    'course3'].order
ICustomUGApplicantEdit['fst_sit_fname'].order = ICustomUGApplicant[
    'fst_sit_fname'].order
ICustomUGApplicantEdit['fst_sit_no'].order = ICustomUGApplicant[
    'fst_sit_no'].order
ICustomUGApplicantEdit['fst_sit_sc_pin'].order = ICustomUGApplicant[
    'fst_sit_sc_pin'].order
ICustomUGApplicantEdit['fst_sit_sc_serial_number'].order = ICustomUGApplicant[
    'fst_sit_sc_serial_number'].order
ICustomUGApplicantEdit['fst_sit_date'].order = ICustomUGApplicant[
    'fst_sit_date'].order
ICustomUGApplicantEdit['fst_sit_type'].order = ICustomUGApplicant[
    'fst_sit_type'].order

class ICustomPGApplicantEdit(ICustomPGApplicant):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomPGApplicantEdit[
    'date_of_birth'].order =  ICustomPGApplicant['date_of_birth'].order
ICustomPGApplicantEdit[
    'email'].order =  ICustomPGApplicant['email'].order

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(IPUTMEApplicantEdit):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(ICustomApplicant):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """

    reg_number = schema.TextLine(
        title = u'Registration Number',
        required = False,
        )

class ICustomApplicantRefereeReport(IApplicantRefereeReport):
    """A referee report.
    """

    duration = schema.Text(
        title = _(u'How long and in what capacity have you known the candidate?'),
        required = False,
        )

    itellectual = schema.Choice(
        title = _(u'Intellectual Capacity'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    persistent = schema.Choice(
        title = _(u'Capacity for Persistent and Independent Academic Study'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    imaginative = schema.Choice(
        title = _(u'Ability for Imaginative Thought'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    productive = schema.Choice(
        title = _(u'Promise of Productive Scholarship'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    previous = schema.Choice(
        title = _(u'Quality of Previous Work'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    expression = schema.Choice(
        title = _(u'Oral and Written Expression in English'),
        required = False,
        readonly = False,
        vocabulary = rating_vocab,
        )

    personality = schema.Text(
        title = _(u'Please comment on the candidate\'s personality '
            'with particular reference to his/her moral character, emotional '
            'and physical stabilty'),
        required = False,
        )

    promise = schema.Choice(
        title = _(u'Candidate\'s overall promise'),
        required = False,
        readonly = False,
        vocabulary = overallpromise_vocab,
        )

    report = schema.Text(
        title = _(u'Any other relevant information which would help '
            'in determining the candidate\'s suitability?'),
        required = False,
        )

    objection = schema.Text(
        title = _(u'Have you any objection to the contents of this '
            'evaluation being disclosed to any award-given body if '
            'the need arises?'),
        required = False,
        )

